#include "stdafx.h"
#include "EKF.h"
#include "mm.h"


CEKF::CEKF(double x, double y, double theta)
{
	X.resize(3,1);
	X(0,0) = x;
	X(1,0) = y;
	X(2,0) = theta;

	P.resize(3,3);
	P.null();
}

CEKF::~CEKF()
{	
} 

void CEKF::predict(double sl, double sr)
{
	double ka = 0.2;
	double kr = 0.05;
	double kl = 0.05;

	double dtheta = (sr - sl)/_axle_length;
	double ds     = (sr + sl)/2;
	
	double th = X(2,0) + dtheta/2;
	double cos_th = cos(th);
	double sin_th = sin(th);

	dMatrix Xp(3,1);

	Xp(0,0) = X(0,0) + ds*cos_th;
	Xp(1,0) = X(1,0) + ds*sin_th;
	Xp(2,0) = X(2,0) + dtheta;
	
	dMatrix A(3,3);
	dMatrix F(3,2);
	dMatrix G(2,2);
	
	A.unit();
	A(0,2) = ka*-ds*sin_th;
	A(1,2) = ka* ds*cos_th;

	double da = ds/(2*_axle_length);
	F(0,0) = 0.5*cos_th - da*sin_th;
	F(1,0) = 0.5*sin_th + da*cos_th;
	F(2,0) =  1./_axle_length;
	F(0,1) = 0.5*cos_th + da*sin_th;
	F(1,1) = 0.5*sin_th - da*cos_th;
	F(2,1) = -1./_axle_length;

	G(0,0) = kr*fabs(sr); 
	G(1,1) = kl*fabs(sl);

	P = A*P*~A + F*G*~F;
	X = Xp;
}

void CEKF::update(double zx, double zy, double r, double a)
{
	double rho = sqrt((zx - X(0,0))*(zx - X(0,0))+(zy - X(1,0))*(zy - X(1,0)));
	double rho2 = rho*rho;

	dMatrix hx(2,1);
	dMatrix H(2,3);
	dMatrix R(2,2);

	hx(0,0) = rho;
	hx(1,0) = DeltaRad(atan2(zy - X(1,0), zx - X(0,0)), X(2,0));

	H(0,0) = -(zx - X(0,0))/rho;
	H(0,1) = -(zy - X(1,0))/rho;
	H(0,2) =  0;
	H(1,0) =  (zx - X(0,0))/rho2;
	H(1,1) = -(zy - X(1,0))/rho2;
	H(1,2) = -1;
	
	R(0,0) = 0.5;
	R(0,1) = 0;
	R(1,0) = 0;
	R(1,1) = 0.5;

	dMatrix K = P*~H*!(H*P*~H + R);

	dMatrix Zk(2,1);

	Zk(0,0) = r;
	//Zk(1,0) = a;
	Zk(1,0) = DeltaRad(a, hx(1,0)) + hx(1,0);

	X = X + K*(Zk - hx);
	P = P - K*H*P;
}